Stan Math Library: stan/math/prim/scal/prob/student_t_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

student_t_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_STUDENT_T_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_STUDENT_T_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/square.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/lbeta.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/digamma.hpp>
17 #include <stan/math/prim/scal/meta/length.hpp>
18 #include <stan/math/prim/scal/fun/grad_reg_inc_beta.hpp>
19 #include <stan/math/prim/scal/fun/inc_beta.hpp>
20 #include <stan/math/prim/scal/meta/include_summand.hpp>
21 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
22 #include <boost/random/student_t_distribution.hpp>
23 #include <boost/random/variate_generator.hpp>
24 #include <cmath>
25
26 namespace stan {
27
28 namespace math {
29
55 template <bool propto, typename T_y, typename T_dof,
56 typename T_loc, typename T_scale>
57 typename return_type<T_y, T_dof, T_loc, T_scale>::type
58 student_t_log(const T_y& y, const T_dof& nu, const T_loc& mu,
59 const T_scale& sigma) {
60 static const char* function("stan::math::student_t_log");
61 typedef typename stan::partials_return_type<T_y, T_dof, T_loc,
62 T_scale>::type
63 T_partials_return;
64
65 using stan::math::check_positive_finite;
66 using stan::math::check_finite;
67 using stan::math::check_not_nan;
68 using stan::math::check_consistent_sizes;
69
70 // check if any vectors are zero length
71 if (!(stan::length(y)
72 && stan::length(nu)
73 && stan::length(mu)
74 && stan::length(sigma)))
75 return 0.0;
76
77 T_partials_return logp(0.0);
78
79 // validate args (here done over var, which should be OK)
80 check_not_nan(function, "Random variable", y);
81 check_positive_finite(function, "Degrees of freedom parameter", nu);
82 check_finite(function, "Location parameter", mu);
83 check_positive_finite(function, "Scale parameter", sigma);
84 check_consistent_sizes(function,
85 "Random variable", y,
86 "Degrees of freedom parameter", nu,
87 "Location parameter", mu,
88 "Scale parameter", sigma);
89
90 // check if no variables are involved and prop-to
91 if (!include_summand<propto, T_y, T_dof, T_loc, T_scale>::value)
92 return 0.0;
93
94 VectorView<const T_y> y_vec(y);
95 VectorView<const T_dof> nu_vec(nu);
96 VectorView<const T_loc> mu_vec(mu);
97 VectorView<const T_scale> sigma_vec(sigma);
98 size_t N = max_size(y, nu, mu, sigma);
99
100 using std::log;
101 using stan::math::digamma;
102 using stan::math::lgamma;
103 using stan::math::square;
104 using stan::math::value_of;
105 using std::log;
106
107 VectorBuilder<include_summand<propto, T_y, T_dof, T_loc, T_scale>::value,
108 T_partials_return, T_dof> half_nu(length(nu));
109 for (size_t i = 0; i < length(nu); i++)
110 if (include_summand<propto, T_y, T_dof, T_loc, T_scale>::value)
111 half_nu[i] = 0.5 * value_of(nu_vec[i]);
112
113 VectorBuilder<include_summand<propto, T_dof>::value,
114 T_partials_return, T_dof> lgamma_half_nu(length(nu));
115 VectorBuilder<include_summand<propto, T_dof>::value,
116 T_partials_return, T_dof>
117 lgamma_half_nu_plus_half(length(nu));
118 if (include_summand<propto, T_dof>::value) {
119 for (size_t i = 0; i < length(nu); i++) {
120 lgamma_half_nu[i] = lgamma(half_nu[i]);
121 lgamma_half_nu_plus_half[i] = lgamma(half_nu[i] + 0.5);
122 }
123 }
124
125 VectorBuilder<!is_constant_struct<T_dof>::value,
126 T_partials_return, T_dof> digamma_half_nu(length(nu));
127 VectorBuilder<!is_constant_struct<T_dof>::value,
128 T_partials_return, T_dof>
129 digamma_half_nu_plus_half(length(nu));
130 if (!is_constant_struct<T_dof>::value) {
131 for (size_t i = 0; i < length(nu); i++) {
132 digamma_half_nu[i] = digamma(half_nu[i]);
133 digamma_half_nu_plus_half[i] = digamma(half_nu[i] + 0.5);
134 }
135 }
136
137 VectorBuilder<include_summand<propto, T_dof>::value,
138 T_partials_return, T_dof> log_nu(length(nu));
139 for (size_t i = 0; i < length(nu); i++)
140 if (include_summand<propto, T_dof>::value)
141 log_nu[i] = log(value_of(nu_vec[i]));
142
143 VectorBuilder<include_summand<propto, T_scale>::value,
144 T_partials_return, T_scale> log_sigma(length(sigma));
145 for (size_t i = 0; i < length(sigma); i++)
146 if (include_summand<propto, T_scale>::value)
147 log_sigma[i] = log(value_of(sigma_vec[i]));
148
149 VectorBuilder<include_summand<propto, T_y, T_dof, T_loc, T_scale>::value,
150 T_partials_return, T_y, T_dof, T_loc, T_scale>
151 square_y_minus_mu_over_sigma__over_nu(N);
152
153 VectorBuilder<include_summand<propto, T_y, T_dof, T_loc, T_scale>::value,
154 T_partials_return, T_y, T_dof, T_loc, T_scale>
155 log1p_exp(N);
156
157 for (size_t i = 0; i < N; i++)
158 if (include_summand<propto, T_y, T_dof, T_loc, T_scale>::value) {
159 const T_partials_return y_dbl = value_of(y_vec[i]);
160 const T_partials_return mu_dbl = value_of(mu_vec[i]);
161 const T_partials_return sigma_dbl = value_of(sigma_vec[i]);
162 const T_partials_return nu_dbl = value_of(nu_vec[i]);
163 square_y_minus_mu_over_sigma__over_nu[i]
164 = square((y_dbl - mu_dbl) / sigma_dbl) / nu_dbl;
165 log1p_exp[i] = log1p(square_y_minus_mu_over_sigma__over_nu[i]);
166 }
167
168 OperandsAndPartials<T_y, T_dof, T_loc, T_scale>
169 operands_and_partials(y, nu, mu, sigma);
170 for (size_t n = 0; n < N; n++) {
171 const T_partials_return y_dbl = value_of(y_vec[n]);
172 const T_partials_return mu_dbl = value_of(mu_vec[n]);
173 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
174 const T_partials_return nu_dbl = value_of(nu_vec[n]);
175 if (include_summand<propto>::value)
176 logp += NEG_LOG_SQRT_PI;
177 if (include_summand<propto, T_dof>::value)
178 logp += lgamma_half_nu_plus_half[n] - lgamma_half_nu[n]
179 - 0.5 * log_nu[n];
180 if (include_summand<propto, T_scale>::value)
181 logp -= log_sigma[n];
182 if (include_summand<propto, T_y, T_dof, T_loc, T_scale>::value)
183 logp -= (half_nu[n] + 0.5)
184 * log1p_exp[n];
185
186 if (!is_constant_struct<T_y>::value) {
187 operands_and_partials.d_x1[n]
188 += -(half_nu[n]+0.5)
189 * 1.0 / (1.0 + square_y_minus_mu_over_sigma__over_nu[n])
190 * (2.0 * (y_dbl - mu_dbl) / square(sigma_dbl) / nu_dbl);
191 }
192 if (!is_constant_struct<T_dof>::value) {
193 const T_partials_return inv_nu = 1.0 / nu_dbl;
194 operands_and_partials.d_x2[n]
195 += 0.5*digamma_half_nu_plus_half[n] - 0.5*digamma_half_nu[n]
196 - 0.5 * inv_nu
197 - 0.5*log1p_exp[n]
198 + (half_nu[n] + 0.5)
199 * (1.0/(1.0 + square_y_minus_mu_over_sigma__over_nu[n])
200 * square_y_minus_mu_over_sigma__over_nu[n] * inv_nu);
201 }
202 if (!is_constant_struct<T_loc>::value) {
203 operands_and_partials.d_x3[n]
204 -= (half_nu[n] + 0.5)
205 / (1.0 + square_y_minus_mu_over_sigma__over_nu[n])
206 * (2.0 * (mu_dbl - y_dbl) / (sigma_dbl*sigma_dbl*nu_dbl));
207 }
208 if (!is_constant_struct<T_scale>::value) {
209 const T_partials_return inv_sigma = 1.0 / sigma_dbl;
210 operands_and_partials.d_x4[n]
211 += -inv_sigma
212 + (nu_dbl + 1.0) / (1.0 + square_y_minus_mu_over_sigma__over_nu[n])
213 * (square_y_minus_mu_over_sigma__over_nu[n] * inv_sigma);
214 }
215 }
216 return operands_and_partials.value(logp);
217 }
218
219 template <typename T_y, typename T_dof, typename T_loc, typename T_scale>
220 inline
221 typename return_type<T_y, T_dof, T_loc, T_scale>::type
222 student_t_log(const T_y& y, const T_dof& nu, const T_loc& mu,
223 const T_scale& sigma) {
224 return student_t_log<false>(y, nu, mu, sigma);
225 }
226 }
227 }
228 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::math::NEG_LOG_SQRT_PI
const double NEG_LOG_SQRT_PI
Definition: constants.hpp:189
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
stan::math::square
fvar< T > square(const fvar< T > &x)
Definition: square.hpp:15
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
grad_reg_inc_beta.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::log1p_exp
fvar< T > log1p_exp(const fvar< T > &x)
Definition: log1p_exp.hpp:13
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
lbeta.hpp
stan::math::log1p
fvar< T > log1p(const fvar< T > &x)
Definition: log1p.hpp:16
stan::math::student_t_log
return_type< T_y, T_dof, T_loc, T_scale >::type student_t_log(const T_y &y, const T_dof &nu, const T_loc &mu, const T_scale &sigma)
The log of the Student-t density for the given y, nu, mean, and scale parameter.
Definition: student_t_log.hpp:58
inc_beta.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.